package hero.entity;

import hero.interfaces.BnProjectHookPK;
import hero.interfaces.BnProjectHookValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnProjectHook
 *
 * @author Oriol Montala
 * @version $Revision: 1.2 $
 *
 * @ejb:bean name="BnProjectHook"
 *           display-name="BnProject Hook Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnProjectHook"
 *           local-jndi-name="ejb/hero/BnProjectHook_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"         
 *
 * @ejb:ejb-ref ejb-name="SequenceGenerator"
 *
 * @ejb:transaction type="Supports"
 *
 * 
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.bean
 *     ejb-name="BnProjectHook"
 *     jndi-name="ejb/hero/BnProjectHook"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnProjectHook"
 * @jonas.cleanup create
 *
 * @ejb:finder signature="java.util.Collection findAll()" 
 *     query="SELECT object(p) FROM BnProjectHook p"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByProject(int projectId)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnProjectHook p Where p.bnProject.id = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnProjectHookLocal findByName(int projectId, java.lang.String name)" view-type="local" 
 *     query="SELECT OBJECT(p) From BnProjectHook p Where p.bnProject.id = ?1 AND p.name = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnProjectHook"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnProjectHookLight"
 *
 * @jboss:table-name table-name="BnProjectHook"
 *
 * @jboss:create-table create="true"
 *
 * @jboss:remove-table remove="false"
 **/
public abstract class BnProjectHookBean
   implements EntityBean
{
   
   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------  
   
   public EntityContext mContext;
   
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------     
   

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------  
   
   /**
   * Retrieve the BnProjectProperty's id.
   *
   * @return Returns an int representing the id of this BnProjectProperty.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"   
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getId();
   
   /**
   * Set the Property's id.
   *
   * @param pId The id of this Property. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( int pId );
   

    
    /** Retrieve the BnProject.
     *
     * @return Returns a BnProjectLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="projecthook" role-name="hook-by" cascade-delete="yes"
     * @jboss:auto-key-fields
     * @jonas:automatic-pk="true"
     * @ejb:transaction type="Supports"
     **/
    public abstract BnProjectLocal getBnProject();
    
    /**
     * Set the BnProject.
     *
     * @ejb:interface-method view-type="local"
     * @param pProject The BnProject of this Hook.
     * @ejb:transaction type="Required" 
     **/
    public abstract void setBnProject( BnProjectLocal pProject );



   /**
   * Retrieve the Name value.
   *
   * @return Returns an String representing the Hook Name value.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();
   
   /**
   * Set the Name value.
   * @ejb:interface-method view-type="both"
   * @param key The Name of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String hookName );

   /**
   * Retrieve the Event value.
   *
   * @return Returns an String representing the Event value.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="event"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getEvent();
   
   /**
   * Set the Event value.
   * @ejb:interface-method view-type="both"
   * @param value The Event value of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setEvent( String hookEvent );


   /**
   * Retrieve the Type value.
   *
   * @return Returns an int representing the Type value.
   * @ejb.value-object
   *    match="light"   
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();
   
   /**
   * Set the Type value.
   * @ejb:interface-method view-type="both"
   * @param value The Type value of this Hook.
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int hookType );

    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Supports"
     */
    public abstract BnProjectHookValue getBnProjectHookValue();
    
    /**
     * @ejb.interface-method 
     * @ejb:transaction type="Required"
     */
    public abstract void setBnProjectHookValue(BnProjectHookValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------  
   
   /**
   * Create a Hook based on the supplied Hook Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnProjectHookPK ejbCreate(BnProjectHookValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       setBnProjectHookValue(prp);

	   try {
		  BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
		  this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.PROJECT_HOOK));
		
	   } catch (BonitaServiceException e) {
		  throw new CreateException(e.getMessage());
	   }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
   * Create a Hook based on the supplied parameters.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnProjectHookPK ejbCreate(String hookName, String hookEvent, int hookType )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(hookName);
       this.setEvent(hookEvent);
       this.setType(hookType);

	   try {
			  BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
			  this.setId(serviceLocator.generateUniqueId(BonitaServiceLocator.PROJECT_HOOK));
		
	   } catch (BonitaServiceException e) {
			  throw new CreateException(e.getMessage());
       }

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }
   
   public void unsetEntityContext()
   {
      mContext = null;
   }
   
   public void ejbActivate()
   {
   }
   
   public void ejbPassivate()
   {
   }
   
   public void ejbLoad()
   {
   }
   
   public void ejbStore()
   {
   }
   
   public void ejbRemove()
      throws
         RemoveException
   {
   }
   
}
