package hero.session;
/*
* 01/02/2004 - 17:04:13
*
* ProjectSessionEJB.java - 
* Copyright (C) 2004 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnIterationLightValue;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.BonitaConfig;
import hero.util.EngineException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.HeroHookException;
import hero.util.BonitaDeadline;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;
import hero.interfaces.UserServiceLocalHome;
import hero.interfaces.UserServiceLocal;

import hero.interfaces.BnNodePerformerAssignLocal;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.util.MailNotification;
import hero.interfaces.Constants;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import bsh.Interpreter;

/**
 * Session Bean Template
 *
 * @ejb:bean name="Engine"
 *           display-name="Engine Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/Engine"
 *           local-jndi-name="ejb/hero/Engine_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Required"
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="Engine"
 *      jndi-name="ejb/hero/Engine"
 **/

public class EngineBean implements SessionBean, Constants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger log = Logger.getLogger("log");

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocal mProject;
	private ProjectSessionLocal mPSession;
	private JMSServicesSessionLocal jms;
	private boolean raiseError = false;
	private ArrayList visited = new ArrayList();
	private Collection iterationPath = new ArrayList();
	private BonitaConfig bonitaConfig;
	
	private BnProjectLocal model=null;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------  
	/**
	 * Terminates the project
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminate(String projectName)
	throws EngineException, HeroException {
		if (this.getProjectConfigLog(projectName))
			log.debug("Start terminate the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome pHome;
		BnProjectLocal pLocal;
		ProjectSessionLocalHome psHome;
		ProjectSessionLocal temp = mPSession; // For subprocess consistence	
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {	
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			pLocal = pHome.findByName(projectName);
			if (pLocal.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot terminate a hidden project !!");
			if (nodesTerminated(pLocal.getBnNodes())) {
				pLocal.setState(hero.interfaces.Constants.Pj.TERMINATED);
				pLocal.setEndDate(new java.util.Date());
			} else {
				throw new EngineException("Project cannot be terminated. Some nodes are still active");
			}
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		if (this.getProjectConfigLog(projectName))
			log.debug(logProjectProperties("Properties after terminated: ")+ ", project = " + projectName+" started by " + mContext.getCallerPrincipal().getName());
		mPSession = temp;
		if (this.getProjectConfigLog(projectName))
			log.debug("End terminate the project = " + projectName+" started by " + mContext.getCallerPrincipal().getName());
	}
	
	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void startActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		if (this.getProjectConfigLog(projectName))
			log.debug("Start startActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot start an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		if (this.getProjectConfigLog(projectName))
			log.debug(logActivityProperties(nodeName, "Properties before start")+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		// if activity performer != null and user == activity performer --> performer assigment case
		// if activity performer is null --> common case
		if ( (nd.getActivityPerformer() != null  && ( nd.getActivityPerformer()).equals(mContext.getCallerPrincipal().getName())) || ! (nd.getActivityPerformer() != null))
		{
			execute(nodeName, hero.interfaces.Constants.Nd.START);
			if (mProject.getState()== hero.interfaces.Constants.Pj.INITIAL)
				mProject.setState(hero.interfaces.Constants.Pj.STARTED);
			if (raiseError == true)
			{
				raiseError = false;
				throw new HeroHookException("An error is produced during the execution of a BeforeStart hook...");
			}
		}
		else
			throw new HeroException("Only activity performer can execute this activity.");
		if (this.getProjectConfigLog(projectName))
			log.debug("End of start activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		if (this.getProjectConfigLog(projectName))
			log.debug(logActivityProperties(nodeName, "Properties after start")+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		
	}
	
	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminateActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		if (this.getProjectConfigLog(projectName))
			log.debug("Start terminateActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot terminate an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			//this.getProjectModel();
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		if (this.getProjectConfigLog(projectName))
			log.debug(logActivityProperties(nodeName, "Properties before terminate")+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		execute(nodeName, hero.interfaces.Constants.Nd.TERMINATE);		
		if (raiseError == true)
		{
			raiseError = false;
			throw new HeroHookException("An error is produced during the execution of a afterTerminate hook...");
		}
		if (this.getProjectConfigLog(projectName))
			log.debug("End of terminate activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		if (this.getProjectConfigLog(projectName))
			log.debug(logActivityProperties(nodeName, "Properties after terminate")+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
	}
	
	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void cancelActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		if (this.getProjectConfigLog(projectName))
			log.debug("Start cancelActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot cancel an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			//this.getProjectModel();
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.CANCEL);
		if (this.getProjectConfigLog(projectName))
			log.debug("End of cancel activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
	}
	
	/**
	 * Suspend the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void suspendActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		//log.debug("suspendActivity: nodeName=" + nodeName);
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.SUSPEND);
	}
	
	/**
	 * Non_Automatic operation method that resume specify activity
	 */
	
	/**
	 * Resume the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void resumeActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		//log.debug("resumeActivity: nodeName=" + nodeName);
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.RESUME);
	}
	
	/**
	 * Active an External Agent
	 * @param agentName the name of the external agent
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void activeAgent(String projectName, String agentName)
	throws EngineException, HeroException {
		//log.debug("activeAgent: agentName=" + agentName);
		BnAgentLocal ag;
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			ag = mProject.getBnAgent(agentName);
			mPSession.setAgentState(ag, hero.interfaces.Constants.Ag.ACTIVE);
			Collection edges = ag.getOutEdges();
			if (!edges.isEmpty())
				modifyAgentEdges(edges);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
	}
	
	// Utility method
	private void execute(String _node, int operation) throws HeroException, EngineException {
		//log.debug("execute: node="+ _node+" operation="+ hero.interfaces.Constants.Nd.operationName[operation]);
		
		BnNodeLocal nd;
		nd = mProject.getBnNode(_node);
		
		int newState = nd.getTransition().computeState(nd, operation);
		if (this.getProjectConfigLog(mPSession.getName()))
			//log.debug("Execute: New node State "+ _node+ " :"+ hero.interfaces.Constants.Nd.nodeStateName[newState]);
			
			if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION)
				throw new EngineException("Bad state transition " + _node);
			
		if (newState != nd.getState()) {
			if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
				if (nd.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE)
				{
					this.terminate(_node);
					mPSession.propagatesSubProcessProperties(_node);
				}
				jms.sendProjectEvent(EventConstants.TERMINATED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORETERMINATE);
				mPSession.propagateNodeProperties(_node);
			} else if (newState == hero.interfaces.Constants.Nd.EXECUTING) {
				if (nd.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE)
					mPSession.propagatesParentProperties(_node);
				jms.sendProjectEvent(EventConstants.START,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORESTART);
			} else if (newState == hero.interfaces.Constants.Nd.ANTICIPATING) {
				jms.sendProjectEvent(EventConstants.ANTICIPATING,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,hero.interfaces.Constants.Nd.ANTICIPATE);
			} else if (newState == hero.interfaces.Constants.Nd.DEAD) {
				jms.sendProjectEvent(EventConstants.CANCELED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node, hero.interfaces.Constants.Nd.ONCANCEL);
			}
			
			if (activityStarted(newState)) {
				this.hookExecute(_node,hero.interfaces.Constants.Nd.AFTERSTART);
			}
			
			// Activities iteration
			if (newState == hero.interfaces.Constants.Nd.TERMINATED && mPSession.getIterationExist(nd.getName()))
			{				
				mPSession.setNodeState(nd,hero.interfaces.Constants.Nd.INITIAL);
				String preNode = mPSession.getToIteration(nd.getName());
				String condition = mPSession.getIterationCondition(nd.getName());
				
				if (preNode != null && (evaluateCondition(condition, nd))==Constants.CONDITION_TRUE)
				{
					this.setIterationPathInitial(nd.getName(),preNode);
					BnNodeLocal preNodeLocal = mProject.getBnNode(preNode);
					mPSession.setNodeState(mProject.getBnNode(preNode),hero.interfaces.Constants.Nd.READY);
					if ( preNodeLocal.getBnNodePerformerAssign() != null)
						this.performAssign(preNodeLocal);					
				}
				else if (preNode != null)
				{
					mPSession.setNodeState(nd,hero.interfaces.Constants.Nd.TERMINATED);
					activeStandByEdges(nd.getName(),preNode);
				}
			}
			else
			{
				mPSession.setNodeState(nd, newState);
				mPSession.setNodeExecutor(_node);
				
				// Deadline handling				
				if (newState==hero.interfaces.Constants.Nd.READY && (nd.getDeadline()!=null ||nd.getRelativeDeadline()!=null))
					BonitaDeadline.getInstance().startDeadline(_node, nd.getBnProject().getId());
				else{
					if ((newState==hero.interfaces.Constants.Nd.TERMINATED || newState==hero.interfaces.Constants.Nd.DEAD) && nd.getDeadline()!=null && nd.getDeadline().getTime()>new Date().getTime())
						BonitaDeadline.getInstance().removeNodeNotification(mProject.getName(),_node,nd.getDeadline().getTime());									
					if ((newState==hero.interfaces.Constants.Nd.TERMINATED || newState==hero.interfaces.Constants.Nd.DEAD) && nd.getRelativeDeadline()!=null && new Date().getTime()+nd.getRelativeDeadline().getTime()>new Date().getTime())					
						BonitaDeadline.getInstance().removeNodeNotification(mProject.getName(),_node,nd.getRelativeDeadline().getTime());
				}
				
				try{
					if (newState==hero.interfaces.Constants.Nd.READY && nd.getBnNodePerformerAssign() != null)
						this.performAssign(nd);
					else if (newState==hero.interfaces.Constants.Nd.READY)
							this.hookExecute(_node,	hero.interfaces.Constants.Nd.ONREADY);
						
				} catch (Exception e){
					e.printStackTrace();
					throw new HeroException("Error when call of performAssign()!!!");
				}
				Collection edges = nd.getOutBnEdges();
				if (edges.isEmpty() && isInstance(mProject.getName()) && newState != Constants.Nd.INITIAL){
					if (!instanceWasModified(nd))
					{
						this.generateOutNodes(nd);
						edges = nd.getOutBnEdges();
					}
				}
				modifyNodeEdges(newState, edges);
			}
		}
		
		if (newState == hero.interfaces.Constants.Nd.TERMINATED)
			this.hookExecute(_node,	hero.interfaces.Constants.Nd.AFTERTERMINATE);
		
		if (newState == hero.interfaces.Constants.Nd.EXECUTING && nd.getType() == hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE)
			this.execute(_node,hero.interfaces.Constants.Nd.TERMINATE);
	}
	
	private void performAssign(BnNodeLocal nd) throws  HeroException {
		try {	
			BnNodePerformerAssignLocal performerAssign = null;
			BnUserLocal mUser;
			UserServiceLocalHome ushome;
			UserServiceLocal ul;
			String creator = mContext.getCallerPrincipal().getName();
			performerAssign = nd.getBnNodePerformerAssign();
			hero.performerAssign.PerformerAssign h = hero.performerAssign.PerformerAssign.make(performerAssign.getName(), performerAssign.getType());
			h.execute(this, performerAssign.getType(), nd, creator);
			
			// send notification to the activityPerformer of the node
			String activityPerformerName = nd.getActivityPerformer();
			
			try {
				BnUserLocalHome userhome=hero.interfaces.BnUserUtil.getLocalHome();
				ushome=hero.interfaces.UserServiceUtil.getLocalHome();
				mUser=userhome.findByName(activityPerformerName);
				
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				throw new HeroException(activityPerformerName+" does not exists");
			}
			try{
				ul= ushome.create();
				Map infos = ul.getUserInfos(mUser.getName());
				if (infos.get("email")!=null)	{
					MailNotification mailNotif = new MailNotification();
					mailNotif.initMailService();									
					mailNotif.sendMail((String)infos.get("email"),"Bonita Notification: activity assigned to you","Hello,\n\n For the Project: " + mProject.getName()+ ",\nthe activity for the node : " + nd.getName() + "\nis ready to be started by you.\n\nPlease connect to the Bonita manager application." );
				}
			} catch (Exception e){
				throw new HeroException("Activity assign Notification fails" + e.getMessage());
			}
			
		} catch (HeroException he) {
			he.printStackTrace();
			throw new HeroException("Error in performerAssignment" + he.getMessage());
		}		 
	}
	
	/**
	 * Method that consult new node state value and modify out edges (state 
	 * propagation) 
	 */
	
	private void modifyNodeEdges(int nodeState, Collection edges)
	throws HeroException, EngineException {
		int newEdgeState;
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			boolean inNode = nodeExistInIteration(edge.getInBnNode().getName());
			boolean outNode = nodeExistInIteration(edge.getOutBnNode().getName());
			if ((inNode && outNode) || (!inNode && !outNode) ||(!inNode && outNode)) // ! StandBy edges (for nodes iteration)
			{
				if (nodeState != hero.interfaces.Constants.Nd.TERMINATED) {
					newEdgeState =hero.interfaces.Constants.edgeTransition[Constants.CONDITION_TRUE][nodeState];
					mPSession.setEdgeState(edge, newEdgeState);
					edgeChanged(edge);
				} else {
					newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(edge.getCondition(),edge.getInBnNode())][nodeState];
					if (edge.getState() != newEdgeState) {
						mPSession.setEdgeState(edge, newEdgeState);
						edgeChanged(edge);	
					}
				}
			}
		}
	}
	
	/**
	 * Method that find the standBy edges of the iteration and active them. 
	 */
	
	private void activeStandByEdges(String lastNode, String firstNode) throws HeroException {
		ArrayList nodes = new ArrayList();
		boolean stop = false; int index=0;
		
		Collection activeEdges = mPSession.getNodeOutEdges(lastNode);
		Iterator j = activeEdges.iterator();
		while (j.hasNext())
		{
			String aEdge = (String)j.next();
			if (!nodes.contains(mPSession.getEdgeOutNode(aEdge)))
			{
				BnEdgeLocal ed = mProject.getBnEdge(aEdge);
				int newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(ed.getCondition(),ed.getInBnNode())][mPSession.getNodeState(lastNode)];
				
				mPSession.setEdgeState(ed, newEdgeState);
				edgeChanged(ed);
			}
		}
		while (!stop)
		{
			Collection edges = mPSession.getNodeInEdges(lastNode);
			Iterator i = edges.iterator();
			while (i.hasNext())
			{
				String edgeName = (String)i.next();
				if (!nodes.contains(mPSession.getEdgeInNode(edgeName)))
					nodes.add(mPSession.getEdgeInNode(edgeName));
			}
			if (nodes.size()==0)
				nodes.add(firstNode);
			String actNode = (String)nodes.get(index);
			activeEdges = mPSession.getNodeOutEdges(actNode);
			j = activeEdges.iterator();
			while (j.hasNext())
			{
				String aEdge = (String)j.next();
				if (!nodes.contains(mPSession.getEdgeOutNode(aEdge)))
				{
					BnEdgeLocal ed = mProject.getBnEdge(aEdge);
					int newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(ed.getCondition(), ed.getInBnNode())][mPSession.getNodeState(actNode)];
					
					mPSession.setEdgeState(ed, newEdgeState);
					edgeChanged(ed);
				}
			}
			lastNode = (String)nodes.get(index);
			if (lastNode.equals(firstNode))
			{
				stop=true;
				nodes.remove(lastNode);
			}
			index++;
		}
	}	
	
	/**
	 * Method that find the existing nodes between lastNode and firstNode) 
	 */
	private boolean nodeExistInIteration(String node) throws HeroException {
		mPSession.initProject(mProject.getModel().getName());
		Collection properties = mPSession.getProperties();		
		Object[] ite = mPSession.getIterations();
		
		//Iterator ite = iterations.iterator();
		for (int i=0; i<ite.length;i++)
		{
			BnIterationLightValue iv = (BnIterationLightValue)ite[i];
			if (isInstance(node))
				node = getModel(node);
			if (isNodeInIterationPath(iv.getFromNode(), iv.getToNode(),node) || node.equals(iv.getFromNode()))
			{
				mPSession.initProject(mProject.getName());
				return true;
			}
		}	
		mPSession.initProject(mProject.getName());
		return false;
	}
	
	private boolean isNodeInIterationPath(String in, String out, String node) throws HeroException {
		if (isInstance(in))
			in = getModel(in);
		if (isInstance(out))
			out = getModel(out);
		if (isInstance(node))
			node = getModel(node);
		BnNodeLocal nd = mProject.getModel().getBnNode(in);
		Collection edges = nd.getInBnEdges();
		Iterator eds = edges.iterator();
		while (eds.hasNext())
		{
			BnEdgeLocal ed = (BnEdgeLocal)eds.next();
			String edgeNode = ed.getInBnNode().getName();
			if (edgeNode.equals(node))
			{
				iterationPath.add(node);
				return true;
			}
			if (edgeNode.equals(out))
				return false;
			else
				if (isNodeInIterationPath(edgeNode,out,node))
					return true;
		}
		return false;
	}
	
	private void setIterationPathInitial(String in, String out) throws HeroException
	{
		ArrayList nodes = new ArrayList();
		BnNodeLocal actualNode = mProject.getBnNode(in);
		if (actualNode.getType()==Nd.SUB_PROCESS_NODE)
			initSubProcess(actualNode.getName());
		nodes.add(actualNode);
		while (!(actualNode.getName()).equals(out) || !nodes.isEmpty())
		{
			nodes.remove(actualNode);
			Collection edges = actualNode.getInBnEdges();
			Iterator eds = edges.iterator();
			while (eds.hasNext())
			{
				BnEdgeLocal ed = (BnEdgeLocal)eds.next();
				BnNodeLocal node = ed.getInBnNode();
				if (isNodeInIterationPath(in, out, node.getName()))
				{
					mPSession.setNodeState(node,Constants.Nd.INITIAL);
					if (node.getType()==Nd.SUB_PROCESS_NODE)
						initSubProcess(node.getName());
					mPSession.setEdgeState(ed,Constants.Ed.INITIAL);
					nodes.add(node);
				}
			}
			if (!nodes.isEmpty())
				actualNode = (BnNodeLocal)nodes.get(0);
			
		}	
	}
	
	/**
	 * Method that propagate the AgentEdges state
	 */
	private void modifyAgentEdges(Collection edges)
	throws HeroException, EngineException {
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			mPSession.setAgentEdgeState(edge,hero.interfaces.Constants.Ed.ANTICIPATING);
			BnNodeLocal nd = edge.getOutBnNode();
			int operation = nd.getActivation().state(nd);
			execute(nd.getName(), hero.interfaces.Constants.Nd.ANTACTIVE);
		}
	}
	
	private void edgeChanged(BnEdgeLocal edge)
	throws HeroException, EngineException {
		BnNodeLocal nd = edge.getOutBnNode();
		if (nd.getState() != hero.interfaces.Constants.Nd.DEAD) {
			int operation = nd.getActivation().state(nd);
			execute(nd.getName(), operation);
		}
	}
	
	private Collection getNodeInEdges(BnNodeLocal nd ){
		Collection edgeValues = new ArrayList();
		Collection edges = nd.getInBnEdges();
		Iterator nodeEdges = edges.iterator();
		while (nodeEdges.hasNext())
			edgeValues.add(((BnEdgeLocal)nodeEdges.next()).getBnEdgeValue());
		return edgeValues;
		
	}
	
	private boolean activityStarted(int state) throws HeroException {
		return (
				state == hero.interfaces.Constants.Nd.ANTICIPATING
				|| state == hero.interfaces.Constants.Nd.EXECUTING);
	}
	
	private void hookExecute(String nodeName, String eventName)
	throws HeroException, HeroHookException {
		Collection hooks;
		Collection interHooks;
		BnNodeLocal node;
		
		try {
			node = mProject.getBnNode(nodeName);
			model = node.getBnProject().getModel();
			
			if (nodeName.matches(".*_instance.*"))
				nodeName=nodeName.substring(0,nodeName.indexOf("_instance"));

			BnNodeLocal nodeLocal;
			try {
				nodeLocal =model.getBnNode(nodeName);
			}catch (HeroException he){nodeLocal = node;}							
					
			hooks = nodeLocal.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					if (this.getProjectConfigLog(mPSession.getName()))
						log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}
			
			interHooks = nodeLocal.getBnInterHooks();
			for (Iterator i = interHooks.iterator(); i.hasNext();) {
				BnNodeInterHookLocal hk = (BnNodeInterHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					if (this.getProjectConfigLog(mPSession.getName()))
						log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}
			
			hooks = model.getBnHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					if (this.getProjectConfigLog(mPSession.getName()))
						log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}
			
			interHooks = model.getBnInterHooks();
			for (Iterator i = interHooks.iterator(); i.hasNext();) {
				BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					if (this.getProjectConfigLog(mPSession.getName()))
						log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}
		}catch(HeroHookException hh){
			if (this.getProjectConfigLog(mPSession.getName()))
				log.error("hookExecute ERROR, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName()+" Error message: "+hh.getMessage());
			if (eventName.equals(hero.interfaces.Constants.Nd.AFTERSTART) || eventName.equals(hero.interfaces.Constants.Nd.BEFORETERMINATE)
					|| eventName.equals(hero.interfaces.Constants.Nd.ANTICIPATE) || eventName.equals(hero.interfaces.Constants.Nd.ONCANCEL)) {
				mContext.setRollbackOnly();
				    throw new HeroHookException("Cannot execute hook "+hh.getMessage());
			}
			else 
				raiseError = true;
		};
	}
	
	private boolean nodesTerminated(Collection nodes) {
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) i.next();
			int state = node.getState();
			if (state != hero.interfaces.Constants.Nd.TERMINATED
					&& state != hero.interfaces.Constants.Nd.EXPIRED
					&& state != hero.interfaces.Constants.Nd.DEAD)
				return false;
		}
		return true;
	}
	
	public int evaluateCondition(String condition, BnNodeLocal node) {
		try {
			if (condition != null) {
				condition = "import hero.interfaces.BnNodeLocal;\n"
					+ "boolean evalCondition (Object node) {\n\n\n"
					+ "return("
					+ condition
					+ ");"
					+ "}";
				Interpreter i = new Interpreter();
				
				// BnNode properties mapping
				
				Collection prop = node.getBnProperties();
				for (Iterator props = prop.iterator(); props.hasNext();) {
					BnNodePropertyLocal pl = (BnNodePropertyLocal) props.next();
					i.set(pl.getTheKey(), pl.getTheValue());
				}
				
				//Project properties mapping
				BnProjectLocal project = node.getBnProject();
				Collection projectProp = project.getBnProperties();
				for (Iterator props=projectProp.iterator();props.hasNext();) {
					BnProjectPropertyLocal pP=(BnProjectPropertyLocal)props.next();
					i.set(pP.getTheKey(),pP.getTheValue());
				}
				i.set("node",node);
				i.eval(condition);
				i.eval("result=evalCondition(node)");
				if (((Boolean) i.get("result")).booleanValue() == true)
					return (Constants.CONDITION_TRUE);
				else
					return (Constants.CONDITION_FALSE);
			} else
				return (Constants.CONDITION_TRUE);
		} catch (Exception t) {
			t.printStackTrace();
			return (Constants.CONDITION_FALSE);
		}
	}
	
	private String logActivityProperties(String nodeName, String prelog)
	throws HeroException {
		Collection np = mPSession.getNodeProperties(nodeName);
		//Collection np = mProject.getBnNode(nodeName).getBnProperties();
		Iterator i = np.iterator();
		String thelog = prelog;
		if (np.size() != 0) {
			while (i.hasNext()) {
				thelog = thelog + " key= '";
				BnNodePropertyValue property = (BnNodePropertyValue) i.next();
				thelog = thelog + property.getTheKey();
				thelog = thelog + "' value = '" + property.getTheValue() + "'";
			}
		} else
			thelog = prelog + " No property";
		return thelog;
	}
	private String logProjectProperties(String prelog) throws HeroException {
		Collection np = mPSession.getProperties();
		Iterator i = np.iterator();
		String thelog = prelog;
		if (np.size() != 0) {
			while (i.hasNext()) {
				thelog = thelog + " key= '";
				BnProjectPropertyValue property =
					(BnProjectPropertyValue) i.next();
				thelog = thelog + property.getTheKey();
				thelog = thelog + "' value = '" + property.getTheValue() + "'";
			}
		} else
			thelog = prelog + " No property";
		return thelog;
	}
	
	// Init activities of the subProcess project (used in subProcess activities iteration)
	private void initSubProcess(String subProcess) throws HeroException {
		ProjectSessionLocalHome psh;
		ProjectSessionLocal psj;
		BnProjectLocalHome ph;
		BnProjectLocal pj;
		try{
			psh = ProjectSessionUtil.getLocalHome();
			psj = psh.create();	
			psj.initProject(subProcess);
			
			ph = BnProjectUtil.getLocalHome();
			pj = ph.findByName(subProcess);
			Collection nodes = pj.getBnNodes();
			Iterator nds = nodes.iterator();
			while (nds.hasNext())
			{
				BnNodeLocal node = (BnNodeLocal)nds.next();
				if (node.getType() == Nd.SUB_PROCESS_NODE)
					this.initSubProcess(node.getName());
				psj.setNodeState(node,hero.interfaces.Constants.Nd.INITIAL);
			}			
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	private void generateOutNodes(BnNodeLocal nd) throws HeroException {
		BnNodeLocal parentNode;
		model = nd.getBnProject().getModel();
		if (nd.getName().matches(".*_instance.*")) // node is a subProcess 
			parentNode = model.getBnNode(this.getModel(nd.getName()));
		else
			parentNode = model.getBnNode(nd.getName());
		Collection outEdges = parentNode.getOutBnEdges();
		Iterator edges = outEdges.iterator();
		while (edges.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) edges.next();
			BnNodeLocal node = edge.getOutBnNode();
			String nodeName = node.getName();
			
			if (node.getType() == Nd.SUB_PROCESS_NODE) // Node is a subprocess
			{
				try {
					BnProjectLocal temp = mProject;
					// Save BnProjectLocal reference before instantiation
					BnProjectLocalHome opHome = BnProjectUtil.getLocalHome();
					nodeName = mPSession.instantiateProject(nodeName);
					// Create instance of the subProcess
					mProject = opHome.findByName(nodeName);
					mProject.setParent(nd.getBnProject().getName());
					// Set subProcess parent
					mProject = temp; // Go back to the instance saved.
					mPSession.initProject(mProject.getName());
				} catch (Exception e) {
					throw new HeroException(e.getMessage());
				}
			}
			if (!this.nodeExist(nodeName))
				mPSession.copyNode(nodeName,node,nd);
			
			//	Copy Node Properties (default value)
			Collection nodeProp = node.getBnProperties();
			Iterator nodeProps = nodeProp.iterator();
			while (nodeProps.hasNext())
			{
				BnNodePropertyLocal ndProp = (BnNodePropertyLocal)nodeProps.next();
				mPSession.setNodeProperty(nodeName,ndProp.getTheKey(),ndProp.getTheValue(),ndProp.getPropagate());				
			}
			
			mPSession.copyEdge(nodeName,nd, edge);
			/*	try {
			 String edgeName = mPSession.addEdge(nd.getName(), nodeName);
			 mPSession.setEdgeCondition(edgeName,edge.getCondition());
			 }catch (hero.util.NodeStartedException nse) {} // Instances execution consistency*/		
			
			// If in activity was automatic propagate properties to new ones
			if (nd.getType() == Constants.Nd.AND_JOIN_AUTOMATIC_NODE || nd.getType() == Constants.Nd.OR_JOIN_AUTOMATIC_NODE)
				mPSession.propagateNodeProperties(nd.getName());
			
			if (!mPSession.getIterationExist(nodeName))
			{
				// Add iteration
				mPSession.initProject(parentNode.getBnProject().getName());
				if (mPSession.getIterationExist(node.getName())) {
					String to = mPSession.getToIteration(node.getName());
					String condition = mPSession.getIterationCondition(node.getName());
					mPSession.initProject(nd.getBnProject().getName());
					to = this.nodeConvertion(to);
					mPSession.addIteration(nodeName, to, condition);
				} else
					mPSession.initProject(nd.getBnProject().getName());
			}
		}
	}
	
	// Get the name of the project model of this instance       
	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}
	
	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}
	
	private String nodeConvertion(String name) {
		Collection nodes = mProject.getBnNodes();
		Iterator nds = nodes.iterator();
		while (nds.hasNext())
		{
			String node = ((BnNodeLocal)nds.next()).getName();
			if (node.matches(name+"_.*"))
				return node;				
		}
		return name;
	}
	
	private boolean nodeExist(String node) throws HeroException{
		return(mPSession.getNodesNames().contains(node));
	}
	
	private boolean getProjectConfigLog(String projectName) throws HeroException{			
		return (bonitaConfig.getProcessLog(projectName));
	}

	private boolean instanceWasModified(BnNodeLocal node) throws HeroException{
		BnProjectLocal model = node.getBnProject().getModel();
		String name = node.getName();
		if (this.isInstance(name))
			name = this.getModel(node.getName());
		try{
			model.getBnNode(name);
		}catch(HeroException e){return true;}
		return false;
	}
	
	/**
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
		try{
			JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			jms=home.create();
		}catch(Exception e){throw new CreateException(e.getMessage());}
	}
	
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
			this.bonitaConfig = new BonitaConfig();
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}
	
	public void ejbRemove() {
	}
	
	public void ejbActivate() {
	}
	
	public void ejbPassivate() {
	}
	
}
