package hero.util;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: BonitaConfigClient.java,v 1.1 2004/11/29 12:29:48 mvaldes Exp $
*
--------------------------------------------------------------------------
*/

import java.net.MalformedURLException;
import java.net.URI;
import java.util.List;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import org.objectweb.carol.util.configuration.CarolCurrentConfiguration;

public class BonitaConfigClient {
	
	private ObjectName config;
    private MBeanServerConnection server;
	//private MBeanServer server;	
	private static BonitaConfigClient bc = null; // Singleton pattern

	public BonitaConfigClient() throws HeroException{	
		try{
			CarolCurrentConfiguration carolConfig = CarolCurrentConfiguration.getCurrent();	
			Properties props = carolConfig.getRMIProperties("jrmp");
			String sCarolURL = props.getProperty(Context.PROVIDER_URL);
			URI carolURL = new URI(sCarolURL);
			String host= carolURL.getHost();
			int portNb = carolURL.getPort();
			String port = String.valueOf(portNb);
			String url = null;
			
			url = "service:jmx:rmi:///jndi/rmi://"+host+":" + port + "/jrmpconnector_jonas";
			// Try to connect to the MBeanServer
			JMXServiceURL connURL = null;
			try {
				connURL = new JMXServiceURL(url);
			} catch (MalformedURLException e) {
				throw new HeroException("Can't create JMXServiceURL with string: " + url);
			}
			JMXConnector connector = null;
			try {
				connector = JMXConnectorFactory.newJMXConnector(connURL, null);
			} catch (MalformedURLException e1) {
				throw new HeroException("there is no provider for the protocol in " + url);
			} catch (java.io.IOException e) {
				throw new HeroException("Connector client cannot be made because of a communication problem (used URL: " + url + ")");
			}
			try {   	    	
				connector.connect(null);
				server = connector.getMBeanServerConnection();    		
			} catch (java.io.IOException ioe) {
				throw new HeroException("connection could not be made because of a communication problem");
			}		
			
			config = new ObjectName("jonas:type=service,name=BonitaConfigService");					
		} catch (Exception e) {e.printStackTrace();
		throw new HeroException(e.getMessage());
		}		
	}
	
	public static BonitaConfigClient getInstance() throws HeroException{
		if ( bc == null) {
			bc = new BonitaConfigClient();
		}
		return bc;
	}
		
	public boolean getProcessJMS(String projectName) throws HeroException
	{
		try {
			Boolean value =(Boolean) server.invoke(config, "getProcessJMS", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value.booleanValue();
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public boolean getProcessLog(String projectName) throws HeroException
	{
		try {
			Boolean value =(Boolean) server.invoke(config, "getProcessLog", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value.booleanValue();
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public boolean getProcessTrace(String projectName) throws HeroException
	{
		try {
			Boolean value =(Boolean) server.invoke(config, "getProcessTrace", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value.booleanValue();
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public String getPocessLogLevel(String projectName) throws HeroException
	{
		try {
			String value =(String) server.invoke(config, "getProcessLogLevel", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value;
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public String getPocessTraceLevel(String projectName) throws HeroException
	{
		try {
			String value =(String) server.invoke(config, "getProcessTraceLevel", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value;
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public String getPocessHistoric(String projectName) throws HeroException
	{
		try {
			String value =(String) server.invoke(config, "getProcessHistoric", new Object[] { projectName},
					new String[] {"".getClass().getName()});
			return value;
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	public void setProcessJMS(String projectName,boolean value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessJMS", new Object[] { projectName,new Boolean(value)},
					new String[] {"".getClass().getName(),"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setProcessLog(String projectName,boolean value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessLog", new Object[] { projectName,new Boolean(value)},
					new String[] {"".getClass().getName(),"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setProcessTrace(String projectName,boolean value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessTrace", new Object[] { projectName,new Boolean(value)},
					new String[] {"".getClass().getName(),"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setProcessTraceLevel(String projectName,String value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessTraceLevel", new Object[] { projectName,value},
					new String[] {"".getClass().getName(),"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setProcessLogLevel(String projectName,String value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessLogLevel", new Object[] { projectName,value},
					new String[] {"".getClass().getName(),"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setPocessHistoric(String projectName,String value) throws HeroException
	{
		try {
			server.invoke(config, "setProcessHistoric", new Object[] { projectName,value},
					new String[] {"".getClass().getName(),"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	
	public void updateJMS(boolean jms) throws HeroException
	{
		try {
			server.invoke(config, "updateJMS", new Object[] {new Boolean(jms)},new String[] {"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void updateLog(boolean log) throws HeroException
	{
		try {
			server.invoke(config, "updateLog", new Object[] {new Boolean(log)},new String[] {"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}		
	}
	
	public void updateTrace(boolean trace) throws HeroException
	{
		try {
			server.invoke(config, "updateTrace", new Object[] {new Boolean(trace)},new String[] {"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}		
	}
	
	public void updateLogLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateLogLevel", new Object[] { value},new String[] {"".getClass().getName()});			
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}		
	}
	
	public void updateTraceLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateTraceLevel", new Object[] { value},new String[] {"".getClass().getName()});			
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}		
	}
	
	public void updateHistoric(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateHistoric", new Object[] { value},new String[] {"".getClass().getName()});			
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}		
	}
	
}