/*
 * Created on 23 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.servlets;

import mc.formgenerator.servlets.util.*;

import hero.util.HeroException;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.chiba.xml.xforms.ChibaContext;
import org.chiba.xml.xforms.exception.XFormsException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import mc.formgenerator.bonita.BonitaActivityAdapter;
import mc.formgenerator.bonita.BonitaActivityExecutor;
import mc.formgenerator.bonita.BonitaProjectAdapter;
import mc.formgenerator.bonita.BonitaProjectExecutor;
import mc.formgenerator.bonita.DocumentParser;
import mc.formgenerator.ggf.GGF;

/**
 * Servlet that will init and launch the form generator
 * @author sempereb
 */
public class ServletBonitaForm extends HttpServlet {

	//The context root
	private String contextRoot = null;

	//GGF instance
	private GGF ggf = null;
	
	//Allow us to know if we are working with an activity or a project
	private Boolean isActivity; 
	
	//Name of the project we are working with
	private String projectName = null;

	//Name of the activity we are working with
	private String activityName = null;
		
	//Application mode consumer or exploitant
	private String mode = null;	

	/**
	 * Start the server side interpretation of the xforms document
	 * @param document the xforms document we have to interprete
	 * @param writer the servlet's writer
	 */
	private void ggfProcessing(Document document, PrintWriter writer) 
		throws 	TransformerConfigurationException, 
				FileNotFoundException, XFormsException, 
				TransformerException, IOException{
					
		//GGF initialization
		this.ggf.initGGF(document);
			
		//GGF launching
		this.ggf.runGGF(writer);	
	}

	
	
	
	/**
	 * Action on doGet
	 * @param writer Response's writer
	 * @param request Request access
	 * @param response servlet response
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws XFormsException
	 * @throws TransformerException
	 * @throws IOException
	 */
	private void writingUI(PrintWriter writer, HttpServletRequest request, HttpServletResponse response) 
		throws 	LoginException, TransformerConfigurationException, 
				FileNotFoundException, XFormsException, TransformerException, 
				IOException, Exception{
		   	
		//Document for server side interpretation
		Document xformsDocument = null;

		//We must get bonita project name
		this.projectName = request.getParameter("projectName");
		
		//We must get bonita activity name eventually
		this.activityName = request.getParameter("activityName");
	
		//Mode consumer or exploitant
		this.mode = request.getParameter("mode");
		
		//Start or stop the activity
		String action = request.getParameter("action");
		
		//Project name parameter is present
		if(this.projectName != null){
			
			//We are working with an activity
			if(this.activityName != null){
			
			    if (action != null){
				    
			        if(action.equals("stop")){
				        
						//Creation of executor
						BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor(); 
						
						//Terminate activity
						bonitaActivityExecutor.endActivity(this.activityName, this.projectName);
						
				        //Redirection to work list
						response.sendRedirect("/formgenerator-1.0/ServletWorkList");
						
						return;	
			        }
				    else{			    
						this.isActivity = new Boolean(true);
						
						//Creation of adapter
						BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter(); 
						
						//Getting process informations via the adapter
						xformsDocument = bonitaActivityAdapter.getActivityData(this.projectName, activityName);
				    }			        
			    }
			    else{			    
					this.isActivity = new Boolean(true);
					
					//Creation of adapter
					BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter(); 
					
					//Getting process informations via the adapter
					xformsDocument = bonitaActivityAdapter.getActivityData(this.projectName, activityName);
			    }
			}
			else{	
					
				//We are working with a project
				this.isActivity = new Boolean(false);
						
				//Creation of adapter
				BonitaProjectAdapter bonitaProjectAdapter = new BonitaProjectAdapter(); 
					
				//Getting process informations via the adapter
				xformsDocument = bonitaProjectAdapter.getProjectData(this.projectName);
			}		
			
			DocumentParser parser = new DocumentParser();
			
			//If the document has properties
			if(parser.hasProperties(xformsDocument)){									
				//Finally starting of interpretation
				this.ggfProcessing(xformsDocument, writer);
			}
			else{
				//Call executor
				this.callExecutor(xformsDocument);
				if (request.getParameter("forward")!=null)
				{
					response.sendRedirect(request.getParameter("forward"));
			        }
				else
				{
	  				//Page redirection
					if(this.isActivity.booleanValue() == true){
					    
					    if(this.mode.equals("consumer"))
					        //Redirection to todo list
							response.sendRedirect("/formgenerator-1.0/ServletActivityToDoList?projectName=" + projectName);
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator-1.0/ServletWorkList");
					}
					else{
					    if(this.mode.equals("consumer"))
					        //Redirection to instances projets
					        response.sendRedirect("/formgenerator-1.0/ServletInstancesProjects");
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator-1.0/ServletWorkList");				        
					}
			        }
			}
		}
		else
			//Message to user
			(new MessageServlet()).writeMessage(writer, "PARAMETER FOR projectName IS MISSING CANNOT PERFORM OPERATION");
	}
	
	
	
	
	/**
	 * Call the correct executor
	 * @param filledDocument document that will be use by the executor
	 * @param mode the application mode: consumer or exploitant
	 */
	private void callExecutor(Document filledDocument) 
		throws 	MalformedURLException, SAXException, IOException, 
				RemoteException, NamingException, CreateException, HeroException{
			
		if(this.isActivity.booleanValue() == true){
			
			//Creation of executor
			BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor(); 
				
			//Terminate the activity
			bonitaActivityExecutor.actOnActivity(filledDocument, this.mode);	
		}
		else{

			//Creation of executor
			BonitaProjectExecutor bonitaProjectExecutor = new BonitaProjectExecutor(); 
				
			//Instanciate project
			bonitaProjectExecutor.startBonitaProject(filledDocument);
		}		
	}
	
	
	
		
	/**
	 * Initializes the servlet.
	 * @param config - the ServletConfig object
	 * @throws javax.servlet.ServletException
	 */
	public void init(ServletConfig config) 
		throws ServletException{
		
		//Superclass initialize
		super.init(config);

		//read some params from web-inf
		this.contextRoot = getServletConfig().getServletContext().getRealPath("");
		if (this.contextRoot == null)
			this.contextRoot = getServletConfig().getServletContext().getRealPath(".");
	}
	
	
	
	
	/**
	 * Servlet reaction on get method:
	 * Display the form generated for filling a valid XML file
	 */
	protected void doGet(
		HttpServletRequest request,
		HttpServletResponse response)
		throws IOException {

		//Creating a session
		HttpSession session = request.getSession(true);

		//write the 'forward' parameter in the session
		if (request.getParameter("forward")!=null)
		    session.setAttribute("forward",request.getParameter("forward"));
		else
		    session.removeAttribute("forward");
				
		//Content type
		response.setContentType("text/html");
			
		//Getting the writer
		PrintWriter out = response.getWriter();
	
		//Getting the Action URI
		String actionURL = getActionURL(request, response);
				
		try {
		    //Instanciation

		    //instanciate the generator with a transformation file
			if (request.getParameter("transform")!=null)
				this.ggf = new GGF(this.contextRoot, actionURL,request.getParameter("transform"));
			//instanciate with the default xsl file
			else
		        this.ggf = new GGF(this.contextRoot, actionURL);
			
			
			//Setting the ggf
			this.ggf = setupGGF(session);
			
			//Build HTML page according to an activity or a project we are working with
			writingUI(out, request, response);
			
			//Stores the GGF in the session
			session.setAttribute("formgenerator.GGF", this.ggf);
			
			//Stores the state of isActivity attribute in the session
			session.setAttribute("formgenerator.isActivity", this.isActivity);
			
			//Stores the projectname in the session
			session.setAttribute("formgenerator.projectName", this.projectName);
			
			//Stores the activity name in the session
			session.setAttribute("formgenerator.activityName", this.activityName);
			
			//Stores the application mode in the session
			session.setAttribute("formgenerator.mode", this.mode);
			
			//Closing writer
			out.close();
			
		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}
	
	
	
	
	
	/**
	 * Servlet reaction on post method:
	 * Submit information from user
	 */
	protected void doPost(
		HttpServletRequest request,
		HttpServletResponse response) 
		throws IOException{
		
		//Getting the writer
		PrintWriter out = response.getWriter();
		
		//Getting the current session
		HttpSession session = request.getSession(true);		

		//Getting GGF from the session
		this.ggf = (GGF)session.getAttribute("formgenerator.GGF");
		
		//Reset the GGF in the session
		session.setAttribute("formgenerator.GGF", this.ggf);
		
		
		//Getting isActivity from the session
		this.isActivity = (Boolean)session.getAttribute("formgenerator.isActivity");
		
		//Set isActivity in the session
		session.setAttribute("formgenerator.isActivity", this.isActivity);		
		
				
		//Getting isActivity from the session
		this.projectName = (String)session.getAttribute("formgenerator.projectName");
		
		//Set the projectname in the session
		session.setAttribute("formgenerator.projectName", this.projectName);
		
		
		//Getting isActivity from the session
		this.activityName = (String)session.getAttribute("formgenerator.activityName");
		
		//Set the projectname in the session
		session.setAttribute("formgenerator.activityName", this.activityName);
				
		this.mode = (String)session.getAttribute("formgenerator.mode");
		
		if ((this.ggf == null)||(this.isActivity == null)||(this.projectName == null))
			//Exception
			throw new IOException("Invalid session - Cannot retrieve ggf or isActivity or projectName in the session");
		else{
			try {
				this.ggf.handleRequest(request);
				
				//Set content type
				response.setContentType("text/html");

				//Getting filled document
				Document filledDocument = this.ggf.chibaBean.getContainer().getDefaultModel().getInstanceDocument("instance");
				
				//Calling the executor
				this.callExecutor(filledDocument);
				
				//Page redirection
			
				//use the forward parameter form the session in order to redirect the output
				String forward = (String)session.getAttribute("forward");
				if ((forward!=null)&&!forward.equals(""))
				    response.sendRedirect(forward);
				
				//use default redirection
				else
					if(this.isActivity.booleanValue() == true){
					    
					    if(this.mode.equals("consumer"))
					        //Redirection to todo list
							response.sendRedirect("/formgenerator-1.0/ServletActivityToDoList?projectName=" + projectName);
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator-1.0/ServletWorkList");
					}
					else{
					    if(this.mode.equals("consumer"))
					        //Redirection to instances projets
					        response.sendRedirect("/formgenerator-1.0/ServletInstancesProjects");
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator-1.0/ServletWorkList");				        
					}
					
					
				//Render result to output
				this.ggf.runGGF(out);
				
				//Close output stream
				out.close();
				
			} catch (Exception e) {
				//Error
				e.printStackTrace();
			}
		}	
	}




	/**
	 * Create, configure and init the GGF which does the actual request processing.
	 * @param session - the Servlet session
	 * @throws XFormsException
	 */
	private GGF setupGGF(HttpSession session) 
		throws XFormsException {
		
		//Map servletMap = (Map)aAdapter.setParameter("httpservlet.adaptermap", new HashMap());
		Map ggfMap = (Map)ggf.setParameter(ChibaContext.SUBMISSION_RESPONSE, new HashMap());
		
		//Puts information in the map
		ggfMap.put(ChibaContext.SESSION_ID, session.getId());

		return ggf;
	}
	
	
	
	
	/**
	 * Return the action URL with jsession
	 * @param request : The request from user
	 * @param response : The response to its request
	 * @return a String object that represents the action URL
	 */
	private String getActionURL(HttpServletRequest request, HttpServletResponse response) {
		
		//Default URL
		String defaultActionURL =
			request.getScheme()
				+ "://"
				+ request.getServerName()
				+ ":"
				+ request.getServerPort()
				+ request.getContextPath()
				+ request.getServletPath();
		String encodedDefaultActionURL = response.encodeURL(defaultActionURL);
		int sessIdx = encodedDefaultActionURL.indexOf(";jsession");
		String sessionId = null;
		if (sessIdx > -1) {
			sessionId = encodedDefaultActionURL.substring(sessIdx);
		}
		
		//Getting action url
		String actionURL = request.getParameter("action_url");
				
		if (actionURL == null) {
			actionURL = encodedDefaultActionURL;
		} else if (null != sessionId) {
			actionURL += sessionId;
		}

		//Encode the URL to allow for session id rewriting
		actionURL = response.encodeURL(actionURL);
		return actionURL;
	}
}